/*
 *
 * Copyright (c) 2011-2016 The University of Waikato, Hamilton, New Zealand.
 * All rights reserved.
 *
 * This file is part of libprotoident.
 *
 * This code has been developed by the University of Waikato WAND
 * research group. For further information please see http://www.wand.net.nz/
 *
 * libprotoident is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * libprotoident is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#include <string.h>

#include "libprotoident.h"
#include "proto_manager.h"
#include "proto_common.h"


static inline bool match_imesh_req(uint32_t data, uint32_t len) {

	if (len != 36 && len != 32)
		return false;
	
        if (MATCH(data, 0x02, 0x00, 0x00, 0x00))
                return true;

	return false;
}

static inline bool match_imesh_resp(uint32_t data, uint32_t len) {
	if (len == 0)
		return true;
	if (len != 36 && len != 32)
		return false;
	
	if (MATCH(data, 0x02, 0x00, ANY, ANY))
		return true;
	return false;
}
/* XXX Not really sure on this one - based on the code from OpenDPI but I
 * can't find any documentation that confirms this */
static inline bool match_imesh_udp(lpi_data_t *data, lpi_module_t *mod UNUSED) {

	if (match_imesh_req(data->payload[0], data->payload_len[0])) {
		if (match_imesh_resp(data->payload[1], data->payload_len[1]))
			return true;
	}
	if (match_imesh_req(data->payload[1], data->payload_len[1])) {
		if (match_imesh_resp(data->payload[0], data->payload_len[0]))
			return true;
	}

	return false;
}

static lpi_module_t lpi_imesh_udp = {
	LPI_PROTO_UDP_IMESH,
	LPI_CATEGORY_P2P,
	"iMesh_UDP",
	6,
	match_imesh_udp
};

void register_imesh_udp(LPIModuleMap *mod_map) {
	register_protocol(&lpi_imesh_udp, mod_map);
}

